@@ -4,7 +4,7 @@ use 5.010_000;
 use strict;
 use warnings;
 
-our $VERSION = '0.1009';
+our $VERSION = '0.1010';
 
 use parent qw(Exporter);
 our @EXPORT_OK = qw(escaped_string html_escape);
@@ -80,7 +80,10 @@ sub _initialize;
 
 sub load_string { # for <input>
     my($self, $string) = @_;
-
+    if(not defined $string) {
+        $self->throw_error("LoadError: Template string is not given");
+    }
+    $self->{string} = $string;
     my $protocode = $self->_compiler->compile($string);
     $self->_initialize($protocode, undef, undef, undef, undef);
     return $protocode;
@@ -90,6 +93,7 @@ sub render_string {
     my($self, $string, $vars) = @_;
 
     local $self->{cache} = 0;
+    local $self->{string};
     $self->load_string($string);
     return $self->render(undef, $vars);
 }
@@ -146,8 +150,7 @@ sub load_file {
     print STDOUT "load_file($file)\n" if _DUMP_LOAD_FILE;
 
     if($file eq '<input>') { # simply reload it
-        return $self->_load_input()
-            // $self->throw_error("LoadError: Template source <input> does not exist");
+        return $self->load_string($self->{string});
     }
 
     my $f = $self->find_file($file, $mtime);
@@ -278,16 +281,6 @@ sub throw_error {
     goto &Carp::croak;
 }
 
-sub dump :method {
-    my($self) = @_;
-    require 'Data/Dumper.pm'; # we don't want to create its namespace
-    my $dd = Data::Dumper->new([$self], ['xslate']);
-    $dd->Indent(1);
-    $dd->Sortkeys(1);
-    $dd->Useqq(1);
-    return $dd->Dump();
-}
-
 # utility functions
 sub escaped_string;
 
@@ -305,6 +298,10 @@ sub html_escape {
     return escaped_string($s);
 }
 
+sub dump :method {
+    goto &Text::Xslate::Util::p;
+}
+
 1;
 __END__
 
@@ -314,7 +311,7 @@ Text::Xslate - High performance template engine
 
 =head1 VERSION
 
-This document describes Text::Xslate version 0.1009.
+This document describes Text::Xslate version 0.1010.
 
 =head1 SYNOPSIS
 
@@ -432,7 +429,7 @@ I<$level> == 0 creates no caches. It's provided for testing.
 
 =item C<< cache_dir => $dir // File::Spec->tmpdir >>
 
-Specifies the directry used for caches.
+Specifies the directory used for caches.
 
 =item C<< function => \%functions >>
 
@@ -600,12 +597,16 @@ Benchmarks:
 
 L<Template::Benchmark>
 
-=head1 ACKNOWLEDGEMENTS
+=head1 ACKNOWLEDGEMENT
 
 Thanks to lestrrat for the suggestion to the interface of C<render()>.
 
 Thanks to tokuhirom for the ideas, feature requests, encouragement, and bug-finding.
 
+Thanks to gardejo for the proposal to the name "template cascading".
+
+Thanks to jjn1056 to the concept of template overlay (now implemented as C<cascade with ...>).
+
 =head1 AUTHOR
 
 Fuji, Goro (gfx) E<lt>gfuji(at)cpan.orgE<gt>
